import sys, getopt

from scapy.supersocket import L3RawSocket
import socket as Socket
from can_id_sheet import VehicleSegment, CANFrame
from ecu_process import GenericECU
from scapy.config import conf
from scapy.contrib.cansocket import CANSocket
from scapy.main import load_contrib
import multiprocessing as mp
import os


def get_filtered_can_socket(vehicle_segment: VehicleSegment):
    filters = [{'can_id': f.value, 'can_mask': 0x7FF} for f in CANFrame if
               f.execution == vehicle_segment]
    # return CANSocket(bustype='socketcan', channel='vcan0', bitrate=250000, can_filters=filters)
    return CANSocket(channel='vcan0', can_filters=filters)


if __name__ == '__main__':
    # CAN related setup
    # load_layer("can")
    # load_contrib('cansocket')

    # protocol setup (CAN or Ethernet)
    protocol = ''
    interface = ''
    argv = sys.argv[1:]
    try:
        opts, args = getopt.getopt(argv, "hp:i:")
    except getopt.GetoptError:
        print('ecu_coordinator.py -p <can/eth>')
        sys.exit(2)
    for opt, arg in opts:
        print("opt, arg", opt, arg)
        if opt == "-h":
            print('ecu_coordinator.py -p <can/eth> -i <eth_interface>')
            sys.exit()
        elif opt == "-p":
            protocol = arg
        elif opt == "-i":
            interface = arg

    if protocol == "can":
        bashCommand = "/bin/bash -c 'sudo modprobe vcan; sudo ip link add name vcan0 type vcan; sudo ip link set dev vcan0 up'"
        os.system(bashCommand)
        conf.contribs['CANSocket'] = {'use-python-can': True}
        load_contrib('cansocket')
        socket = CANSocket(channel='vcan0')
    else:
        socket = Socket.socket(Socket.AF_PACKET, Socket.SOCK_RAW)
        socket.bind((interface, 0))

    # multiprocessing setup
    mp.set_start_method('fork')
    processes = [
        GenericECU(protocol=protocol,
                   interface=interface,
                   tx_socket=socket,
                   rx_socket=get_filtered_can_socket(VehicleSegment.Chassis) if protocol == 'can' else socket,
                   vehicle_segment=VehicleSegment.Chassis),
        GenericECU(protocol=protocol,
                   interface=interface,
                   tx_socket=socket,
                   rx_socket=get_filtered_can_socket(VehicleSegment.PowerTrain) if protocol == 'can' else socket,
                   vehicle_segment=VehicleSegment.PowerTrain),
        GenericECU(protocol=protocol,
                   interface=interface,
                   tx_socket=socket,
                   rx_socket=get_filtered_can_socket(VehicleSegment.Body) if protocol == 'can' else socket,
                   vehicle_segment=VehicleSegment.Body)
    ]

    for p in processes:
        p.start()

    for p in processes:
        p.join()
